#ifndef LIBCFOTO_BLITZ_H
#define LIBCFOTO_BLITZ_H

#include "constants.h"

/** @defgroup Blitzlicht
 *
 * Berechnungen zur Blitzlichtfotografie.
 *
 * @author Elmar Baumann <eb@elmar-baumann.de>
 * @date 1996/05/31
 *
 *  @{
 */

/**
 * Berechnet die Entfernung des Blitzgerts zum Motiv fr eine bestimmte
 * Blende.
 *
 * @param  lz  Leitzahl des Blitzgerts.
 *             Bedingung: Grer als Null
 * @param  k   Blende.
 *             Bedingung: Grer als Null
 * @param  g   Rckgabe der Entfernung des Blitzgerts
 * @return TRUE, wenn die Entfernung berechnet wurde
 */
BOOL blitz_entfernung_anhand_blende(double lz, double k, double* g);

/**
 * Berechnet die Blende fr eine Entfernung des Blitzgerts zum Motiv.
 *
 * @param  lz  Leitzahl des Blitzgerts.
 *             Bedingung: Grer als Null
 * @param  g   Entfernung des Blitzgerts zum Motiv.
 *             Bedingung: Grer als Null
 * @param  k   Rckgabe der einzustellenden Blende
 * @return TRUE, wenn die Blende berechnet wurde
 */
BOOL blitz_blende_anhand_entfernung(double lz, double g, double* k);

/**
 * Berechnet die Gesamtleitzahl zweier Blitzleuchten aus gleicher Richtung und
 * gleicher Entfernung.
 *
 * @param  lz_1    Leitzahl des ersten Blitzgerts.
 *                 Bedingung: Grer als Null
 * @param  lz_2    Leitzahl des zweiten Blitzgerts.
 *                 Bedingung: Grer als Null
 * @param  lz_ges  Rckgabe der Gesamtleitzahl
 * @return TRUE, wenn die Leitzahl berechnet wurde
 */
BOOL blitz_gesamt_leitzahl(double lz_1, double lz_2, double* lz_ges);

/**
 * Berechnet die Gesamtleitzahl zweier Blitzleuchten aus gleicher Richtung und
 * unterschiedlicher Entfernung.
 *
 * @param  k_1     Blende fr richtige Belichtung beim ersten Blitzgert.
 *                 Bedingung: Grer als Null
 * @param  k_2     Blende fr richtige Belichtung beim zweiten Blitzgert.
 *                 Bedingung: Grer als Null
 * @param  lz_ges  Rckgabe der Gesamtleitzahl
 * @return TRUE, wenn die Gesamtleitzahl berechnet wurde
 */
BOOL blitz_gesamt_leitzahl_anhand_blenden(double k_1, double k_2, double* lz_ges);

/**
 * Berechnet die Blitzanzahl bei Offenblitzmethode, vorgegebener Blende
 * und vorgegebener Entfernung des Blitzgerts zum Motiv.
 *
 * @param  lz      Leitzahl des Blitzgerts.
 *                 Bedingung: Grer als Null
 * @param  k       Blende.
 *                 Bedingung: Grer als Null
 * @param  g       Entfernung des Blitzgerts zum Motiv.
 *                 Bedingung: Grer als Null
 * @param  anzahl  Rckgabe der erforderlichen Blitzanzahl
 * @return TRUE, wenn die Anzahl berechnet wurde
 */
BOOL blitz_anzahl_offenblitz(double lz, double k, double g, double* anzahl);

/**
 * Berechnet die Entfernung des Blitzgerts zum Motiv bei Auszugsverlngerungen
 * und vorgegebener Blende.
 *
 * @param  lz    Leitzahl des Blitzgerts.
 *               Bedingung: Grer als Null
 * @param  k     Blende.
 *               Bedingung: Grer als Null
 * @param  beta  Abbildungsmassstab.
 *               Bedingung: Grer als Null
 * @param  g     Rckgabe der Entfernung des Blitzgerts zum Motiv
 * @return TRUE, wenn die Entfernung berechnet wurde
 */
BOOL blitz_entfernung_nahaufnahme(double lz, double k, double beta, double* g);

/** @} */

#endif
