#ifndef LIBCFOTO_OPTIK_H
#define LIBCFOTO_OPTIK_H

#include "constants.h"

/** @defgroup Optik Optik Berechnungen zur Optik in der Fotografie
 *
 * @author Elmar Baumann <eb@elmar-baumann.de>
 * @date   1996/05/31
 *
 *  @{
 */

/** @name Abbildungsmastab
 */
//@{

/**
 * Berechnet den Abbildungsmastab anhand der Gegenstandsweite.
 *
 * @param  f     Brennweite.
 *               Bedingung: Grer als Null
 * @param  g     Gegenstandsweite
 *               Bedingung: Grer als die Brennweite
 * @param  beta  Rckgabe des Abbildungsmastabs
 * @return TRUE, wenn der Abbildungsmastab berechnet wurde
 */
BOOL optik_abbm_anhand_g(double f, double g, double* beta);

//@}

/** @name Bildweite
 */
//@{

/**
 * Berechnet die Bildweite in Abhngigkeit vom Abbildungsmastab.
 *
 * @param  f     Brennweite.
 *               Bedingung: Grer als Null
 * @param  beta  Abbildungsmastab.
 *               Bedingung: Grer als Null
 * @param  b     Rckgabe der Bildweite
 * @return TRUE, wenn die Bildweite berechnet wurde
 */
BOOL optik_bildweite_anhand_abbm(double f, double beta, double* b);

/**
 * Berechnet die Bildweite in Abhngigkeit von der Gegenstandsweite.
 *
 * @param  f  Brennweite.
 *            Bedingung: Grer als Null
 * @param  g  Gegenstandsweite.
 *            Bedingung: Grer als die Brennweite
 * @param  b  Rckgabe der Bildweite
 * @return TRUE, wenn die Bildweite berechnet wurde
 */
BOOL optik_bildweite_anhand_g(double f, double g, double* b);

//@}

/** @name Gegenstandsweite
 */
//@{

/**
 * Berechnet die Gegenstandsweite in Abhngigkeit vom Abbdildungsmastab.
 *
 * @param  f     Brennweite.
 *               Bedingung: Grer als Null
 * @param  beta  Abbildungsmastab.
 *               Bedingung: Grer als Null
 * @param  g     Rckgabe der Gegenstandsweite
 * @return TRUE, wenn die Gegenstandsweite berechnet wurde
 */
BOOL optik_g_anhand_abbm(double f, double beta, double* g);

/**
 * Berechnet die Gegenstandsweite in Abhngigkeit von der Bildweite.
 *
 * @param  f  Brennweite.
 *            Bedingung: Grer als Null
 * @param  b  Bildweite.
 *            Bedingung: Grer als f
 * @param  g  Rckgabe der Gegenstandsweite
 * @return TRUE, wenn die Gegenstandsweite berechnet wurde
 */
BOOL optik_g_anhand_bildweite(double f, double b, double* g);

/**
 * Berechnet anhand der Gegenstandsweite die Entfernungseinstellung.
 *
 * <p>Die Entfernungseinstellung ist der Abstand von der Sensorebene (Filmebene)
 * zur Einstellebene (Gegenstandsebene).
 *
 * <p>So kann in allen anderen Funktionen die Gegenstandsweite
 * ersetzt werden durch die Entfernungseinstellung eines Objektivs.
 *
 * @param  e  Entfernungseinstellung.
 *            Bedingung: Ger als Null, ger als d und es muss eine Abbildung
 *            entstehen knnen, d.h., die resultierende Gegenstandsweite muss
 *            ger sein als die Brennweite.
 *
 *            <p>Aktuell werden nur Entfernungen akzeptiert, die maximal den
 *            Abbildungsmastab 1 : 1 ergeben. Ist der Hauptebenen-Abstand Null,
 *            ist das der Fall bei 4-facher Brennweite (Minimum: e == 4 * f).
 * @param  d  Hauptebenen-Abstand. Null, wenn unbekannt.
 *            Bedingung: Grer gleich Null und kleiner als e.
 * @param  f  Brennweite.
 *            Bedingung: Grer als Null
 * @param  g  Rckgabe der Gegenstandsweite.
 * @return TRUE, wenn die Gegenstandsweite berechnet wurde
 */
BOOL optik_g_anhand_entfernungs_einstellung(double e, double d, double f, double* g);

//@}

/** @name Blende
 */
//@{

/**
 * Berechnet die Frderliche Blende.
 *
 * @param  z       Zerstreuungskreisdurchmesser.
 *                 Bedingung: Grer als Null
 * @param  beta    Abbildungsmastab.
 *                 Bedingung: Grer als Null
 * @param  lambda  Lichtwellenlnge.
 *                 Bedingung: Grer als Null
 * @param  k_f     Rckgabe der frderlichen Blende
 * @return TRUE, wenn die frderliche Blende berechnet wurde
 */
BOOL optik_foerderliche_blende(double z, double beta, double lambda, double* k_f);

/**
 * Berechnet die effektive Blende, Objektiv in Normalstellung.
 *
 * @param  k     Blendenkennziffer.
 *               Bedingung: Grer als Null
 * @param  beta  Abbildungsmastab.
 *               Bedingung: Grer als Null
 * @param  k_e   Rckgabe der effektiven Blende
 * @return TRUE, wenn die effektive Blende berechnet wurde
 */
BOOL optik_effektive_blende(double k, double beta, double* k_e);

/**
 * Berechnet die effektive Blende unter Bercksichtigung des Pupillenmastabs,
 * Objektiv in Normalstellung.
 *
 * @param  k       Blendenkennziffer.
 *                 Bedingung: Grer als Null
 * @param  beta    Abbildungsmastab.
 *                 Bedingung: Grer als Null
 * @param  beta_p  Pupillenmastab.
 *                 Bedingung: Grer als Null
 * @param  k_e     Rckgabe der effektiven Blende
 * @return TRUE, wenn die effektive Blende berechnet wurde
 */
BOOL optik_effektive_blende_p(double k, double beta, double beta_p, double* k_e);

/**
 * Berechnet die effektive Blende unter Bercksichtigung des Pupillenmastabs,
 * Objektiv in Retrostellung.
 *
 * @param  k       Blendenkennziffer.
 *                 Bedingung: Grer als Null
 * @param  beta    Abbildungsmastab.
 *                 Bedingung: Grer als Null
 * @param  beta_p  Pupillenmastab.
 *                 Bedingung: Grer als Null
 * @param  k_e     Rckgabe der effektiven Blende
 * @return TRUE, wenn die effektive Blende berechnet wurde
 */
BOOL optik_effektive_blende_p_r(double k, double beta, double beta_p, double* k_e);

/**
 * Berechnet die neue Blendenkennziffer bei einer bestimmten
 * Lichtmengennderung.
 *
 * @param  k         Alte Blendenkennziffer.
 *                   Bedingung: Grer als Null
 * @param  delta_li  Faktor der Lichtmengennderung.
 *                   Bedingung: Grer als Null
 * @param  k_neu     Rckgabe der neuen Blendenkennziffer
 * @return TRUE, wenn die Blende berechnet wurde
 */
BOOL optik_blende_anhand_lichtmengen_faktor(double k, double delta_li, double* k_neu);

/**
 * Liefert die neue Blende, wenn der Pupillenmastab bekannt ist.
 *
 * @param  k       Alte Blendenkennziffer.
 *                 Bedingung: Grer als Null
 * @param  beta_p  Pupillenmastab.
 *                 Bedingung: Grer als Null
 * @param  k_neu   Rckgabe der neuen Blende
 * @return TRUE, wenn die neue Blende berechnet wurde
 */
BOOL optik_blende_pup_masstab(double k, double beta_p, double* k_neu);

//@}

/** @name Pupillenmastab
 */
//@{

/**
 * Berechnet den Pupillenmastab.
 *
 * @param  ap      Durchmesser der Austrittspupille.
 *                 Bedingung: Grer als Null
 * @param  ep      Durchmesser der Eintrittspupille.
 *                 Bedingung: Grer als Null
 * @param  beta_p  Rckgabe des Pupillenmastabs
 * @return TRUE, wenn der Pupillenmastab berechnet wurde
 */
BOOL optik_pupillen_masstab(double ap, double ep, double* beta_p);

//@}

/** @name Belichtung
 */
//@{

/**
 * Berechnet den Belichtungszeitverlngerungsfaktor bei Nahaufnahmen
 * anhand der Bildweite.
 *
 * @param  b       Bildweite.
 *                 Bedingung: Grer als f
 * @param  f       Brennweite.
 *                 Bedingung: Grer als Null
 * @param  faktor  Rckgabe des Belichtungszeitverlngerungsfaktors
 * @return TRUE, wenn der Faktor berechnet wurde
 */
BOOL optik_t_faktor_anhand_bildweite(double b, double f, double* faktor);

/**
 * Berechnet den Belichtungszeitverlngerungsfaktor bei Nahaufnahmen
 * anhand des Abbildungsmastabs.
 *
 * @param  beta    Abbildungsmastab.
 *                 Bedingung: Grer als Null
 * @param  f       Brennweite.
 *                 Bedingung: Grer als Null
 * @param  faktor  Rckgabe des Belichtungszeitverlngerungsfaktors
 * @return TRUE, wenn der Faktor berechnet wurde
 */
BOOL optik_t_faktor_anhand_abbm(double beta, double f, double* faktor);

/**
 * Berechnet den Belichtungszeitverlngerungsfaktor bei Nahaufnahmen
 * anhand des Abbildungsmastabs unter Bercksichtigung des
 * Pupillenmastabs.
 *
 * @param  beta    Abbildungsmastab.
 *                 Bedingung: Grer als Null
 * @param  beta_p  Pupillenmastab.
 *                 Bedingung: Grer als Null
 * @param  faktor  Rckgabe des Belichtungszeitverlngerungsfaktors
 * @return TRUE, wenn der Faktor berechnet wurde
 */
BOOL optik_t_faktor_anhand_abbm_p(double beta, double beta_p, double* faktor);

/**
 * Berechnet den Belichtungszeitverlngerungsfaktor bei Nahaufnahmen
 * anhand des Abbildungsmastabs unter Bercksichtigung des Pupillenmastabs,
 * Objektiv in Retrostellung.
 *
 * @param  beta    Abbildungsmastab.
 *                 Bedingung: Grer als Null
 * @param  beta_p  Pupillenmastab.
 *                 Bedingung: Grer als Null
 * @param  faktor  Rckgabe des Belichtungszeitverlngerungsfaktors
 * @return TRUE, wenn der Faktor berechnet wurde
 */
BOOL optik_t_faktor_anhand_abbm_p_r(double beta, double beta_p, double* faktor);

/**
 * Berechnet die Anzahl der Blendenstufen einer Lichtmengennderung.
 * Verlngert sich z.B. die Verschlusszeit um den Faktor 8 (t_faktor),
 * sind das 3 Blendenstufen (k_stufen).
 *
 * @param  t_faktor  Lichtmengennderung
 *                   Bedingung: Grer Null
 * @param  k_stufen  Rckgabe der Blendenstufen
 * @return TRUE bei Erfolg
 */
BOOL optik_k_stufen_anhand_t_faktor(double t_faktor, double* k_stufen);

//@}

/** @name Lochkamera
 */
//@{

/**
 * Berechnet den optimalen Lochdurchmesser bei Lochkameras.
 *
 * @param  lambda  Lichtwellenlnge.
 *                 Bedingung: Grer Null
 * @param  b       Bildweite = Abstand des Lochs zum Film.
 *                 Bedingung: Grer als Null
 * @param  g       Gegenstandsweite.
 *                 Bedingung: Grer als Null
 * @param  d       Rckgabe des optimalen Lochdurchmessers
 * @return TRUE, wenn der Durchmesser berechnet wurde
 */
BOOL optik_loch_durchmesser_opt(double lambda, double b, double g, double* d);

//@}

/** @name Dioptrien
 */
//@{

/**
 * Berechnet die Dioptrien einer Brennweite.
 *
 * @param  f     Brennweite.
 *               Bedingung: Grer als Null
 * @param  diop  Rckgabe der Dioptrien
 * @return TRUE, wenn die Dioptrien berechnet wurden
 */
BOOL optik_dioptrien(double f, double* diop);

/**
 * Berechnet die Brennweite einer Dioptrien-Zahl.
 *
 * @param  diop  Dioptrien-Zahl.
 *               Bedingung: Grer als Null
 * @param  f     Rckgabe der Brennweite
 * @return TRUE, wenn die Brennweite berechnet wurde
 */
BOOL optik_brennweite_anhand_dioptrien(double diop, double* f);

//@}

/** @name Nahlinsen
 */
//@{

/**
 * Berechnet den Abbildungsmastab bei vorgesetzter Nahlinse,
 * Objektiv auf Unendlich eingestellt.
 *
 * @param  f     Brennweite des Objektivs.
 *               Bedingung: Grer als Null
 * @param  diop  Dioptrien der Nahlinse.
 *               Bedingung: Grer als Null
 * @param  beta  Rckgabe des Abbildungsmastabs
 * @return TRUE, wenn der Mastab berechnet wurde
 */
BOOL optik_abbm_nahlinse_u(double f, double diop, double* beta);

/**
 * Berechnet den Abbildungsmastab bei vorgesetzter Nahlinse,
 * Objektiv nicht auf Unendlich eingestellt.
 *
 * @param  f     Brennweite des Objektivs.
 *               Bedingung: Grer als Null
 * @param  diop  Dioptrien der Nahlinse.
 *               Bedingung: Grer als Null
 * @param  g     Gegenstandsweite.
 *               Bedingung: Grer als f
 * @param  beta  Rckgabe des Abbildungsmastabs
 * @return TRUE, wenn der Mastab berechnet wurde
 */
BOOL optik_abbm_nahlinse(double f, double diop, double g, double* beta);

/**
 * Berechnet die Gegenstandsweite bei vorgesetzter Nahlinse,
 * Objektiv auf Unendlich eingestellt.
 *
 * @param  diop  Dioptrien der Nahlinse.
 *               Bedingung: Grer als Null
 * @param  g     Rckgabe der Gegenstandsweite
 * @return TRUE, wenn die Gegenstandsweite berechnet wurde
 */
BOOL optik_g_nahlinse_u(double diop, double* g);

/**
 * Berechnet die Gegenstandsweite bei vorgesetzter Nahlinse,
 * Objektiv nicht auf Unendlich eingestellt.
 *
 * @param  diop   Dioptrien der Nahlinse.
 *                Bedingung: Grer als Null
 * @param  g      Gegenstandsweite.
 *                Bedingung: Grer als Null
 * @param  g_neu  Rckgabe der neuen Gegenstandsweite
 * @return TRUE, wenn die Gegenstandsweite berechnet wurde
 */
BOOL optik_g_nahlinse(double diop, double g, double* g_neu);

//@}

/** @name Projektion
 */
//@{

/**
 * Berechnet die Breite des eines projizierten Bilds auf der Projektionsflche
 * (Bildwand).
 *
 * @param  f       Brennweite des Projektionsobjektivs.
 *                 Bedingung: Grer als Null
 * @param  g       Gegenstandsweite (ca. Abstand Projektor zu Bildwand).
 *                 Bedingung: Grer als f
 * @param  l       Kantenlnge Bilds (Dia: Sichtbares Bild).
 *                 Bedingung: Grer als Null
 * @param  breite  Rckgabe der projizierten Breite
 * @return TRUE, wenn die Breite berechnet wurde
 */
BOOL optik_breite_bildwand(double f, double g, double l, double* breite);

//@}

/** @name Lichtfarbe
 */
//@{

/**
 * Berechnet den Miredwert aus Kelvin.
 *
 * @param  kelvin  Kelvinwert.
 *                 Bedingung: Grer als Null
 * @param  mired   Rckgabe des Miredwerts
 * @return TRUE, wenn die Mired berechnet wurden
 */
BOOL optik_mired(double kelvin, double* mired);

/**
 * Berechnet den Kelvinwert aus Mired.
 *
 * @param  mired   Miredwert.
 *                 Bedingung: Grer als Null
 * @param  kelvin  Rckgabe des Kelvinwerts
 * @return TRUE, wenn der Kelvinwert berechnet wurde
 */
BOOL optik_kelvin(double mired, double* kelvin);

/**
 * Berechnet Miredwert eines Konversionsfilters .
 *
 * Ein negatitiver Wert bedeutet, der Konversionsfilter hat eine
 * eine bluliche Farbe, ein positiver Wert ist eine rtliche Farbe.
 *
 * @param  mired_film    Miredwert des Films (Aufnahmematerials).
 *                       Bedingung: Grer als Null
 * @param  mired_lichtq  Miredwert der Lichtquelle.
 *                       Bedingung: Grer als Null
 * @param  mired_konv    Rckgabe des Miredwerts des Konversionsfilters
 * @return TRUE, wenn die Mired berechnet wurden
 */
BOOL optik_konversions_filter(double mired_film, double mired_lichtq, double* mired_konv);

//@}

/** @name Bildwinkel
 */
//@{

/**
 * Berechnet den Objektivbildwinkel in Grad.
 *
 * <p>Es wird in der Regel die Diagonale (sqrt(lnge+breite)) oder
 * eine Kantenlnge als Bildlnge bergeben.
 *
 * @param  f           Brennweite.
 *                     Bedingung: Grer als Null
 * @param  bildlaenge  Kantenlnge des Bilds auf dem Sensor (Film).
 *                     Bedingung: Grer als Null
 * @param  bildwinkel  Rckgabe des Objektivbildwinkels in Grad
 * @return TRUE, wenn der Bildwinkel berechnet wurde
 */
BOOL optik_bildwinkel(double f, double bildlaenge, double* bildwinkel);

//@}

/** @name Lichtwert
 */
//@{

/**
 * Berechnet den Lichtwert.
 *
 * @param  k   Blendenkennziffer.
 *             Bedingung: Grer als Null
 * @param  t   Verschlusszeit in Sekunden.
 *             Bedingung: Grer als Null
 * @param  ev  Rckgabe des Lichtwerts
 * @return TRUE, wenn der Lichtwert berechnet wurde
 */
BOOL optik_ev(double k, double t, double* ev);

//@}

/** @} */

#endif
