#ifndef LIBCFOTO_STIEFE_H
#define LIBCFOTO_STIEFE_H

#include "constants.h"

/** @defgroup Schaerfentiefe
 *
 * Berechnungen zur Schrfentiefe in der Fotografie.
 *
 * @author Elmar Baumann <eb@elmar-baumann.de>
 * @date 1996/05/31
 *
 *  @{
 */

/**
 * Berechnet den Gesamtbereich der Schrfentiefe.
 *
 * @param  f        Brennweite.
 *                  Bedingung: Grer als Null
 * @param  k        Blende.
 *                  Bedingung: Grer als Null
 * @param  z        Zerstreuungskreisdurchmesser.
 *                  Bedingung: Grer als Null
 * @param  g        Gegenstandsweite.
 *                  Bedingung: Grer als f und kleiner stiefe_hyp()
 * @param  bereich  Rckgabe des Gesamtbereichs der Schrfentiefe
 * @return TRUE, wenn die Schrfentiefe berechnet wurde
 * @see    stiefe_bereich_abbm()
 */
BOOL stiefe_bereich(double f, double k, double z, double g, double* bereich);

/**
 * Berechnet den Nahpunkt der Schrfentiefe, das ist die nheste
 * Gegenstandsweite vor der eingestellten Gegenstandsweite, die noch
 * ausreichend scharf abgebildet wird.
 *
 * @param  f         Brennweite.
 *                   Bedingung: Grer als Null
 * @param  k         Blende.
 *                   Bedingung: Grer als Null
 * @param  z         Zerstreuungskreisdurchmesser.
 *                   Bedingung: Grer als Null
 * @param  g         Gegenstandsweite.
 *                   Bedingung: Grer als f
 * @param  nahpunkt  Rckgabe des Nahpunkts der Schrfentiefe
 * @return TRUE, wenn der Nahpunkt berechnet wurde
 *
 * @see stiefe_nahpunkt_abbm()
 */
BOOL stiefe_nahpunkt(double f, double k, double z, double g, double* nahpunkt);

/**
 * Berechnet den Fernpunkt der Schrfentiefe: Fernste Gegenstandsweite
 * hinter der eingestellten Gegenstandsweite, die noch ausreichend scharf
 * abgebildet wird.
 *
 * @param  f          Brennweite.
 *                    Bedingung: Grer als Null
 * @param  k          Blende.
 *                    Bedingung: Grer als Null
 * @param  z          Zerstreuungskreisdurchmesser.
 *                    Bedingung: Grer als Null
 * @param  g          Gegenstandsweite.
 *                    Bedingung: Grer als Null und kleiner als stiefe_hyp()
 * @param  fernpunkt  Rckgabe des Fernpunkts der Schrfentiefe
 * @return TRUE, wenn der Fernpunkt berechnet wurde
 * @see    stiefe_fernpunkt_abbm()
 */
BOOL stiefe_fernpunkt(double f, double k, double z, double g, double* fernpunkt);

/**
 * Berechnet die hyperfokale Distanz: Gegenstandsweite, bei der die Schrfentiefe
 * von der Hlfte dieser Gegenstandsweite bis Unendlich reicht.
 *
 * @param  f    Brennweite.
 *              Bedingung: Grer als Null
 * @param  k    Blende.
 *              Bedingung: Grer als Null
 * @param  z    Zerstreuungskreisdurchmesser.
 *              Bedingung: Grer als Null
 * @param  hyp  Rckgabe der hyperfokalen Distanz
 * @return TRUE, wenn die hyperfokale Distanz berechnet wurde
 */
BOOL stiefe_hyp(double f, double k, double z, double* hyp);

/**
 * Berechnet die Schrfentiefe anhand des Abbildungsmastabs.
 *
 * @param  beta     Abbildungsmastab.
 *                  Bedingung: Grer als Null
 * @param  f        Brennweite.
 *                  Bedingung: Grer als Null
 * @param  k        Blende.
 *                  Bedingung: Grer als Null
 * @param  z        Zerstreuungskreisdurchmesser.
 *                  Bedingung: Grer als Null
 * @param  bereich  Rckgabe des Bereichs der Schrfentiefe
 * @return TRUE, wenn die Schrfentiefe berechnet wurde
 * @see    stiefe_bereich()
 */
BOOL stiefe_bereich_abbm(double beta, double f, double k, double z, double* bereich);

/**
 * Berechnet die Schrfentiefe anhand des Abbildungsmastabs.
 *
 * <p>Im Gegensatz zu stiefe_bereich_abbm() wird hier die Brennweite ignoriert,
 * was ausreichend genau ist bei Abbildungsmastben ab 0,1 und grer.
 *
 * @param  beta     Abbildungsmastab.
 *                  Bedingung: Grer als Null
 * @param  k        Blende.
 *                  Bedingung: Grer als Null
 * @param  z        Zerstreuungskreisdurchmesser.
 *                  Bedingung: Grer als Null
 * @param  bereich  Rckgabe des Bereichs der Schrfentiefe
 * @return TRUE, wenn die Schrfentiefe berechnet wurde
 * @since  2004/03/06
 */
BOOL stiefe_bereich_abbm_2(double beta, double k, double z, double* bereich);

/**
 * Berechnet den Nahpunkt der Schrfentiefe anhand des Abbildungsmastabs.
 *
 * @param  beta      Abbildungsmastab.
 *                   Bedingung: Grer als Null
 * @param  f         Brennweite.
 *                   Bedingung: Grer als Null
 * @param  k         Blende.
 *                   Bedingung: Grer als Null
 * @param  z         Zerstreuungskreisdurchmesser.
 *                   Bedingung: Grer als Null
 * @param  nahpunkt  Rckgabe des Nahpunkts der Schrfentiefe
 * @return TRUE, wenn der Nahpunkt berechnet wurde
 * @see    stiefe_nahpunkt()
 */
BOOL stiefe_nahpunkt_abbm(double beta, double f, double k, double z, double* nahpunkt);

/**
 * Berechnet den Fernpunkt der Schrfentiefe: Fernste Gegenstandsweite
 * hinter der eingestellten Gegenstandsweite, die noch ausreichend scharf
 * abgebildet wird.
 *
 * @param  beta       Abbildungsmastab.
 *                    Bedingung: Grer als Null
 * @param  f          Brennweite.
 *                    Bedingung: Grer als Null
 * @param  k          Blende.
 *                    Bedingung: Grer als Null
 * @param  z          Zerstreuungskreisdurchmesser.
 *                    Bedingung: Grer als Null
 * @param  fernpunkt  Rckgabe des Fernpunkts der Schrfentiefe
 * @return TRUE, wenn der Fernpunkt berechnet wurde
 * @see    stiefe_fernpunkt()
 */
BOOL stiefe_fernpunkt_abbm(double beta, double f, double k, double z, double* fernpunkt);

/**
 * Liefert die Blende fr eine gewnschte Schrfentiefe.
 *
 * @param  f  Brennweite.
 *            Bedingung: Grer als Null
 * @param  g  Gegenstandsweite.
 *            Bedingung: Grer als  f
 * @param  z  Zerstreuungskreisdurchmesser.
 *            Bedingung: Grer als Null
 * @param  t  Schrfentiefe.
 *            Bedingung: Grer als Null
 * @param  k  Rckgabe der Blende
 * @return TRUE, wenn die Blende berechnet wurde
 */
BOOL stiefe_blende(double f, double g, double z, double t, double* k);

/** @} */

#endif
